# Here's an example parameter that will recieve
# a controller node through the OCI's ENC:
#
#---
#classes:
#   oci_controller:
#      is_first_master: true
#      first_master: zigo-controller-node-3.infomaniak.ch
#      first_master_ip: 192.168.100.40
#      vip_hostname: zigo-api.infomaniak.ch
#      vip_ipaddr: 192.168.101.2
#      network_ipaddr: 192.168.101.0
#      network_cidr: 24
#      other_masters:
#         - zigo-controller-node-4.infomaniak.ch
#         - zigo-controller-node-5.infomaniak.ch
#      other_masters_ip:
#         - 192.168.100.41
#         - 192.168.100.32
#      all_masters:
#         - zigo-controller-node-3.infomaniak.ch
#         - zigo-controller-node-4.infomaniak.ch
#         - zigo-controller-node-5.infomaniak.ch
#      all_masters_ip:
#         - 192.168.100.40
#         - 192.168.100.41
#         - 192.168.100.32
#
# This is re-used in the oci_controller class below
#
class oci::controller(
  $machine_hostname         = undef,
  $machine_ip               = undef,
  $is_first_master          = false,
  $first_master             = undef,
  $first_master_ip          = undef,
  $other_masters            = undef,
  $other_masters_ip         = undef,
  $vip_hostname             = undef,
  $vip_ipaddr               = undef,
  $network_ipaddr           = undef,
  $network_cidr             = undef,
  $all_masters              = undef,
  $all_masters_ip           = undef,
  $all_masters_ids          = undef,
  $has_subrole_db           = true,
  $has_subrole_messaging    = true,
  $has_subrole_api_keystone = true,
  $use_ssl                  = true,
  $rabbit_env               = {},
  $pass_keystone_adminuser  = undef,
  $pass_keystone_db         = undef,
  $pass_keystone_messaging  = undef,
  $pass_nova_db             = undef,
  $pass_glance_db           = undef,
  $pass_neutron_db          = undef,
  $pass_cinder_db           = undef,
  $pass_nova_messaging      = undef,
  $pass_cinder_messaging    = undef,
  $pass_neutron_messaging   = undef,
){

  # Add a virtual IP address
  class { 'corosync':
    authkey                  => '/var/lib/puppet/ssl/certs/ca.pem',
    bind_address             => $machine_ip,
    cluster_name             => 'mycluster',
    enable_secauth           => true,
    set_votequorum           => true,
    quorum_members           => $all_masters_ip,
    quorum_members_ids       => $all_masters_ids,
    log_stderr               => false,
    log_function_name        => true,
    syslog_priority          => 'debug',
    debug                    => true,
  }
  corosync::service { 'pacemaker':
    version => '0',
  }->
  cs_property { 'stonith-enabled':
    value   => 'false',
  }->
  cs_property { 'no-quorum-policy':
    value   => 'stop',
  }->
  cs_primitive { 'openstack-api-vip':
    primitive_class => 'ocf',
    primitive_type  => 'IPaddr2',
    provided_by     => 'heartbeat',
    parameters      => { 'ip' => $vip_ipaddr, 'cidr_netmask' => $network_cidr, 'nic' => 'eth0:0' },
    operations      => { 'monitor' => { 'interval' => '10s' } },
  }

  # Add haproxy that will listen on that virtual IP, and load balance
  # to the different API daemons using tcp mode (as the APIs will do
  # full SSL already).

  # First, we accept binding on non-local IPs:
  sysctl::value { 'net.ipv4.ip_nonlocal_bind':
    value => "1",
    target => '/etc/sysctl.d/ip-nonlocal-bind.conf',
  }->
  # Then we setup HA proxy to bind on that IP
  file { "/etc/haproxy/ssl":
    ensure                  => directory,
    owner                   => 'root',
    mode                    => '0755',
    selinux_ignore_defaults => true,
    require       => Package['haproxy'],
  }->
  file { "/etc/haproxy/ssl/private":
    ensure                  => directory,
    owner                   => 'root',
    mode                    => '0755',
    selinux_ignore_defaults => true,
  }->
  file { "/etc/haproxy/ssl/private/oci-pki-api.pem":
    ensure                  => present,
    owner                   => "haproxy",
    source                  => "/etc/ssl/private/oci-pki-api.pem",
    selinux_ignore_defaults => true,
    mode                    => '0600',
  }
  class { 'haproxy':
    global_options   => {
      'maxconn' => '40960',
      'user'    => 'haproxy',
      'group'   => 'haproxy',
      'daemon'  => '',
    },
    defaults_options => {
      'mode'    => 'http',
    }
  }

  # keystone public
  haproxy::frontend { 'keystonepubfe':
    mode      => 'http',
    bind      => { "${vip_ipaddr}:5000" => ['ssl', 'crt', '/etc/haproxy/ssl/private/oci-pki-api.pem'] },
    options   => {
      'default_backend' => 'keystonepubbe',
    }
  }->
  haproxy::backend { 'keystonepubbe':
    options => [
       { 'mode' => 'http' },
       { 'balance' => 'roundrobin' },
    ],
  }
  haproxy::balancermember { 'keystonepubbm':
    listening_service => 'keystonepubbe',
    ipaddresses       => $all_masters_ip,
    server_names      => $all_masters,
    ports             => 5000,
    options           => 'check ssl verify none',
  }

  # keystone admin
  haproxy::frontend { 'keystoneadmfe':
    mode      => 'http',
    bind      => { "${vip_ipaddr}:35357" => ['ssl', 'crt', '/etc/haproxy/ssl/private/oci-pki-api.pem'] },
    options   => {
      'default_backend' => 'keystoneadmbe',
    }
  }->
  haproxy::backend { 'keystoneadmbe':
    options => [
       { 'mode' => 'http' },
       { 'balance' => 'roundrobin' },
    ],
  }
  haproxy::balancermember { 'keystoneadmbm':
    listening_service => 'keystoneadmbe',
    ipaddresses       => $all_masters_ip,
    server_names      => $all_masters,
    ports             => 35357,
    options           => 'check ssl verify none',
  }

  # Setup databases
  if $has_subrole_db {

    class { 'galera':
      galera_servers      => $all_masters_ip,
      galera_master       => $first_master,
      package_ensure      => 'latest',
      mysql_package_name  => 'mariadb-server',
      client_package_name => 'default-mysql-client',
      vendor_type         => 'mariadb',
      root_password       => 'a_big_secret',
      status_password     => 'a_big_secret',
      configure_repo      => false,
      configure_firewall  => false,
      galera_package_name => 'galera-3',
      override_options => {
        'mysqld' => {
          'bind_address'                    => '0.0.0.0',
          'wait_timeout'                    => '28800',
          'interactive_timeout'             => '30',
          'connect_timeout'                 => '30',
          'character_set_server'            => 'utf8',
          'collation_server'                => 'utf8_general_ci',
        }
      }
    }->
    class { '::keystone::db::mysql':
      dbname   => 'keystonedb',
      password => $pass_keystone_db,
    }
  }

  # RabbitMQ
  if $has_subrole_messaging {
    if $use_ssl {
      $ssl_cert = "/etc/ssl/certs/ssl-cert-snakeoil.pem"
      $ssl_key  = "/etc/ssl/private/ssl-cert-snakeoil.key"
    } else {
      $ssl_cert = 'UNSET'
      $ssl_key  = 'UNSET'
    }

    class { '::rabbitmq':
      package_ensure           => 'latest',
      delete_guest_user        => true,
      ssl                      => $use_ssl,
      ssl_only                 => $use_ssl,
      ssl_cert                 => $ssl_cert,
      ssl_key                  => $ssl_key,
      environment_variables    => $rabbit_env,
      repos_ensure             => false,
      # Clustering options...
      config_cluster           => true,
      cluster_nodes            => $all_masters,
      cluster_node_type        => 'ram',
      erlang_cookie            => 'XAFSRAFNNFJOFSHLEUFT',
      wipe_db_on_cookie_change => true,
    }->
    rabbitmq_vhost { '/':
      provider => 'rabbitmqctl',
      require  => Class['::rabbitmq'],
    }->
    rabbitmq_user { 'keystone':
      admin    => true,
      password => $pass_keystone_messaging,
    }->
    rabbitmq_user_permissions { "keystone@/":
      configure_permission => '.*',
      write_permission     => '.*',
      read_permission      => '.*',
      provider             => 'rabbitmqctl',
      require              => Class['::rabbitmq'],
    }
  }

  # Get memcached
  class { '::memcached':
    listen_ip => $::fqdn,
    udp_port  => 0,
  }

  # Configure Apache to use mod-wsgi-py3, not mod-wsgi
  if $has_subrole_api_keystone {
    # Set python3 for mod-wsgi
    include ::apache::params
    class { '::apache':
      mod_packages => merge($::apache::params::mod_packages, {
        'wsgi' => 'libapache2-mod-wsgi-py3',
      })
    }
  }

  if $has_subrole_api_keystone {
    class { '::keystone::client': }
    class { '::keystone::cron::token_flush': }
    class { '::keystone::cron::fernet_rotate':
      hour   => '*',
      minute => '*/10',
    }

    file { "/etc/keystone/ssl":
      ensure                  => directory,
      owner                   => 'root',
      mode                    => '0755',
      selinux_ignore_defaults => true,
      require       => Package['keystone'],
    }->
    file { "/etc/keystone/ssl/private":
      ensure                  => directory,
      owner                   => 'root',
      mode                    => '0755',
      selinux_ignore_defaults => true,
    }->
    file { "/etc/keystone/ssl/public":
      ensure                  => directory,
      owner                   => 'root',
      mode                    => '0755',
      selinux_ignore_defaults => true,
    }->
    file { "/etc/keystone/ssl/private/${::fqdn}.key":
      ensure                  => present,
      owner                   => "keystone",
      source                  => "/etc/ssl/private/ssl-cert-snakeoil.key",
      selinux_ignore_defaults => true,
      mode                    => '0600',
    }->
    file { "/etc/keystone/ssl/public/${::fqdn}.crt":
      ensure                  => present,
      owner                   => "keystone",
      source                  => '/etc/ssl/certs/ssl-cert-snakeoil.pem',
      selinux_ignore_defaults => true,
      mode                    => '0644',
      notify        => Service['httpd'],
    }

    ensure_resource('file', '/etc/apache2/sites-available/wsgi-keystone.conf', {
      'ensure'  => 'present',
      'content' => '',
      require   => Package['keystone'],
    })

    class { '::keystone':
      debug                      => true,
      database_connection        => "mysql://keystone:${pass_keystone_db}@127.0.0.1/keystonedb",
      catalog_type               => 'sql',
      admin_token                => $pass_keystone_adminuser,
      admin_password             => $pass_keystone_adminuser,
      enabled                    => true,
      service_name               => 'httpd',
      enable_ssl                 => $use_ssl,
      public_bind_host           => "${::fqdn}",
      admin_bind_host            => "${::fqdn}",
      manage_policyrcd           => true,
      enable_credential_setup    => false,
      token_expiration           => 600,
      admin_endpoint             => "https://${vip_hostname}:35357/",
      public_endpoint            => "https://${vip_hostname}:5000/",
      default_transport_url      => os_transport_url({
        'transport' => 'rabbit',
        'host'      => $::fqdn,
        'port'      => '5671',
        'username'  => 'keystone',
        'password'  => $pass_keystone_messaging,
      }),
      notification_transport_url => os_transport_url({
        'transport' => 'rabbit',
        'host'      => $::fqdn,
        'port'      => '5671',
        'username'  => 'keystone',
        'password'  => $pass_keystone_messaging,
      }),
      rabbit_use_ssl             => $use_ssl,
      rabbit_ha_queues           => true,
    }
    include ::apache
    class { '::keystone::wsgi::apache':
      bind_host       => $machine_ip,
      admin_bind_host => $machine_ip,
      ssl             => $use_ssl,
      ssl_key         => "/etc/keystone/ssl/private/${::fqdn}.key",
      ssl_cert        => "/etc/keystone/ssl/public/${::fqdn}.crt",
      workers         => 2,
    }
    class { '::keystone::roles::admin':
      email    => 'production@infomaniak.com',
      password => $pass_keystone_adminuser,
    }->
    class { '::keystone::endpoint':
      public_url => "https://${vip_hostname}:5000/",
      admin_url  => "https://${vip_hostname}:35357/",
    }
    class { '::keystone::disable_admin_token_auth': }
    class { '::openstack_extras::auth_file':
      password       => $pass_keystone_adminuser,
      project_domain => 'default',
      user_domain    => 'default',
      auth_url       => "https://${vip_hostname}:5000/v3/",
    }
    keystone_role { 'creator':
      ensure => present,
    }
    keystone_role { 'member':
      ensure => present,
    }
  }
}
